import sys

# Copied from https://github.com/coderanger/supervisor-stdout/blob/master/setup.py

def write_stdout(s):
    sys.stdout.write(s)
    sys.stdout.flush()

def write_stderr(s):
    sys.stderr.write(s)
    sys.stderr.flush()

def event_handler(event, response):
    line, data = response.split('\n', 1)
    headers = dict([x.split(':') for x in line.split()])
    lines = data.split('\n')
    prefix = '%s %s | '%(headers['processname'], headers['channel'])
    print('\n'.join([prefix + l for l in lines]))



import logging
import unittest
from pymacaron_async.serialization import model_to_task_arg
from pymacaron_async.serialization import task_arg_to_model
from pymacaron_core.swagger.api import API


log = logging.getLogger(__name__)


yaml = """
swagger: '2.0'
info:
  version: '0.0.1'
host: some.server.com
schemes:
  - http
produces:
  - application/json
definitions:
  Foo:
    type: object
    properties:
      foo:
        type: string
      bars:
        type: array
        items:
          $ref: '#/definitions/Bar'

  Bar:
    type: object
    properties:
      baz:
        type: number

"""


class Test(unittest.TestCase):


    def test__model_to_task_arg(self):
        api = API('somename', yaml_str=yaml)
        for o in (1, 's', [1, 2], {'a': 4}):
            self.assertEqual(model_to_task_arg(o), o)

        Foo = api.model.Foo(
            foo='blabla',
            bars=[
                api.model.Bar(baz=2),
                api.model.Bar(baz=3),
            ]
        )

        j = model_to_task_arg(Foo)
        self.assertEqual(
            j,
            {
                '__pymacaron_model_name': 'Foo',
                'bars': [{'baz': 2}, {'baz': 3}],
                'foo': 'blabla',
            }
        )


    def test__task_arg_to_model(self):
        for o in (1, 's', [1, 2], {'a': 4}):
            self.assertEqual(task_arg_to_model(o), o)

        o = task_arg_to_model(
            {
                '__pymacaron_model_name': 'Foo',
                'bars': [{'baz': 2}, {'baz': 3}],
                'foo': 'blabla',
            }
        )

        j = o.to_json()
        self.assertEqual(
            j,
            {
                'bars': [{'baz': 2}, {'baz': 3}],
                'foo': 'blabla',
            }
        )



import logging
from pymacaron_core.models import PyMacaronModel
from pymacaron_core.models import get_model


log = logging.getLogger(__name__)


def model_to_task_arg(o):
    """Take an object, and if it is a PyMacaron model, serialize it into json so it
    can be passed as argument of a Celery task. Otherwise return the object
    unchanged
    """

    if not isinstance(o, PyMacaronModel):
        return o

    # It's a PyMacaron model, let's serialize it to json and attach a magic marker to it
    j = o.to_json()
    j['__pymacaron_model_name'] = o.get_model_name()
    log.debug("Serialized PyMacaronModel %s to celery task arg" % j['__pymacaron_model_name'])
    return j


def task_arg_to_model(o):
    """The opposite of model_to_tupple"""
    if type(o) is dict and '__pymacaron_model_name' in o:
        model_name = o['__pymacaron_model_name']
        log.debug("Deserialized celery task arg to PyMacaronModel: %s" % model_name)
        del o['__pymacaron_model_name']
        return get_model(model_name).from_json(o)
    else:
        return o
